c      BPKM_Matsui_HU.for
c
c      This is the source code program for the model simulations used in the following papers: 
c
c      **************************************************************************************************************   
c      *                                                                                                            *
c      *   The follwoing paper should be referenced in publications which use this code.                           *
c      *                                                                                                            *  
c      **************************************************************************************************************
c
c      Yoshihiko Matsui, Naoya Ando, Hiroshi Sasaki, Taku Matsushita, Koichi Ohno, 
c      Branched pore kinetic model analysis of geosmin adsorption on super-powdered activated carbon, 
c      Water Research, Volume 43, Issue 12, July 2009, Pages 3095-3103.
c
c      The execution file may be obtained from the corresponding author of the papers. 
c
c      This gives time change of adsorbate concentration in batch adsorption of single solute system. 
c      film mass transport (kf), surface diffusion (Ds) , pore diffusion (Dp) , and micro pore diffusion (ks)
c      are the main kinetic parameters
c      adsorption isotherm are described by Freundlich equation 
c      The execution requires 
c                an input file named "BPKM_input.txt" 
c          and a file which contains W and B matrixes.
c
c
c         Yoshihiko Matsui
c         Hokkaido University, 1.5.2009 
c
c	    Orthogonal Collocation method
c              This program requires IMSL subroutine " divpag "
c
	 IMPLICIT DOUBLE PRECISION (A-H,O-Z)

       character colnum*9, ndim4char*2

       parameter (mxparm=50)
	 parameter (ndim4=29)

	 parameter (ndim6=13)
	 parameter (ndim5=ndim4*ndim6*2)

       dimension param(mxparm),y(ndim5)
c	 -------for divpag----------
	 dimension aaa(1,1)
c      -------------------------
       
      dimension acdose(ndim6),acdia(ndim6),radi(ndim6)
      dimension wr(ndim4),br(ndim4,ndim4)

	common/b1/nac
	common/b2/acdose,acdia,radi
      common/b3/ps,rfk,rn,pfk,rkf,rds,rdp,rks,co,ceff,rceff
      common/b4/ncol
      common/b5/wr,br
	                    
      external fcn2,fcnj,umach,sset
      
	open(10,file='BPKM_input.txt',status='old')
      open(20,file='BPKM_output.txt',status='unknown')

      write(ndim4char,'(I2)') ndim4
      colnum = 'col'//ndim4char// '.txt'
 	write(*,*) 'Using '//colnum     
	      
      open(11,file=colnum,status='old')

       read(10,*) miter
	 read(10,*) tol
	 read(10,*) param11
       read(10,*) ntmax
       read(10,*) tmax
       read(10,*) dose
	 read(10,*) nac
       read(10,*) (acdose(i), i=1,nac)
       read(10,*) (acdia(i), i=1,nac)
       read(10,*) ps
       read(10,*) rfk
       read(10,*) rn
       read(10,*) rkf
       read(10,*) rds
	 read(10,*) rdp
	 read(10,*) rks
	 read(10,*) pfk
	 read(10,*) co
c
       write(20,*) miter
	 write(20,*) tol
	 write(20,*) param11
       write(20,*) ntmax
       write(20,*) tmax
	 write(20,*) dose
	 write(20,*) nac
       write(20,*) (acdose(i), i=1,nac)
       write(20,*) (acdia(i), i=1,nac)
       do 301 i=1,nac
       acdose(i)=acdose(i)*dose
	 radi(i)=acdia(i)/2.0
301    continue
       write(20,*) ps
       write(20,*) rfk
       write(20,*) rn
       write(20,*) rkf
       write(20,*) rds
	 write(20,*) rdp 
      write(20,*) rks
	 write(20,*) pfk 
	 write(20,*) co
      write(20,*)
	 	  
       read(11,*) ncol
	 write(20,*) ncol
	 read(11,1003) (wr(i),i=1,ncol)
 1003	 format(4f35.20)
       write(20,1004) (wr(i),i=1,ncol)
 1004  format(10f8.3)
	 do 400 i=1,ncol
	 read(11,1003) (br(i,j),j=1,ncol)
	 write(20,1005) (br(i,j),j=1,ncol)
 1005	 format(10f8.1)
  400	 continue
	 write(20,*)

	 nequ=ndim5

	 close(10)
       close(11)

       rfk=rfk*pfk

       dtmax=tmax/dfloat(ntmax)

       write(*,*) 'solving RKG'
         
        t=0.0

        do 505 i=1,nequ
        y(i)=0.0
  505   continue
        
            istep=0
            rceff=1.0
            write(*,*) t,rceff
            write(20,*) t,rceff
  
          tol=tol
          call sset(mxparm,0.0,param,1)
	    param(1)=1e-10
		param(4)=10000000

c  ------for divpag----------------
	    param(10)=2
	    param(11)=param11

c		Select Gear's BDF method
          param(12)=2

c         MITER=1   for user-provided Jacobian
c         MITER=3   for Jacobian replaced matrics based on by directional derivative
          param(13)=miter

c         'A' matrix is not used, system is explicit
          param(19)=0
c ---------------------------------
       
          ido=1
   50     continue
          istep=istep+1
          tend=istep*dtmax

c          call divprk (ido,nequ,fcn2,t,tend,tol,param,y)
          call divpag (ido,nequ,fcn2,fcnj,aaa,t,tend,tol,param,y)

          if(istep.le.ntmax) then

 	      ceff=co
	      do 503 k=1,nac
	      wrq=0.0
		  do 501 j=1,ncol
	      jj=j+ncol*(k-1)
	      wrq=wrq+wr(j)*y(jj)
	      wrq=wrq+wr(j)*y(jj+ncol*nac)
  501       continue
            ceff=ceff-3.0*acdose(k)*wrq
  503       continue         

		  rceff=ceff/co

            write(*,*) t,rceff
            write(20,*) t,rceff
       
            if(istep.eq.ntmax) ido=3
            go to 50   
          else
          end if
                    
            close(20)
 
            stop
            end


       subroutine fcn2 (nn, t, y, yprime)
       IMPLICIT DOUBLE PRECISION (A-H,O-Z)

	 parameter (ndim4=29)
 	 parameter (ndim6=13)
	      
       dimension y(nn),yprime(nn)
       dimension acdose(ndim6),acdia(ndim6),radi(ndim6)
       dimension wr(ndim4),br(ndim4,ndim4)
       dimension q(ndim4,ndim6),p(ndim4,ndim6),cx(ndim4,ndim6)
       dimension dq(ndim4,ndim6),dp(ndim4,ndim6) 
	 	 
	 common/b1/nac
	 common/b2/acdose,acdia,radi
       common/b3/ps,rfk,rn,pfk,rkf,rds,rdp,rks,co,ceff,rceff
       common/b4/ncol
       common/b5/wr,br

        do 110 i=1,nn
	 yprime(i)=0.0
 110   continue

 	    ceff=co
 	    do 503 k=1,nac
		  wrq=0.0
		  do 501 j=1,ncol
	      jj=j+ncol*(k-1)
	      q(j,k)=y(jj)
		  wrq=wrq+wr(j)*q(j,k)
		  if (pfk.lt.0.9999) then
	      p(j,k)=y(jj+nac*ncol)		   
		  wrq=wrq+wr(j)*p(j,k)
	      else
	      end if
  501       continue

            ceff=ceff-3.0*acdose(k)*wrq 
  503       continue  
    	 
	 do 99  k=1,nac
	 do 101 j=1,ncol
 	  cx(j,k)=dsign( (abs(q(j,k))/rfk)**rn, q(j,k) )
  101  continue    
   99  continue

	 ncol1=ncol-1   
	 do 107 m=1,nac
	 do 104 j=1,ncol1
	 dq(j,m)=0.0

	 do 105	k=1,ncol
	 dq(j,m)=dq(j,m)+br(j,k)*q(k,m)*rds/radi(m)**2
     &   	            +br(j,k)*cx(k,m)*rdp/radi(m)**2/ps
  105  continue
    
       if (pfk.lt.0.9999) then
       dq(j,m)=dq(j,m)-rks*(q(j,m)/pfk-p(j,m)/(1.0-pfk))
       dp(j,m)=        rks*(q(j,m)/pfk-p(j,m)/(1.0-pfk))
	 else
	 end if

  104  continue

	 dq(ncol,m)=rkf/ps/wr(ncol)/3.0*(6.0/acdia(m))*(ceff-cx(ncol,m))
	
	  do 106 j=1,ncol1
	  dq(ncol,m)=dq(ncol,m)-wr(j)/wr(ncol)*dq(j,m)
        if (pfk.lt.0.9999) 
     &  dq(ncol,m)=dq(ncol,m)-wr(j)/wr(ncol)*dp(j,m)
  106   continue

        if (pfk.lt.0.9999) then
        dq(ncol,m)=dq(ncol,m)-rks*(q(ncol,m)/pfk-p(ncol,m)/(1.0-pfk))
        dp(ncol,m)=           rks*(q(ncol,m)/pfk-p(ncol,m)/(1.0-pfk))
	  else
	  end if

  107   continue

  	 do 603 k=1,nac
	      do 601 j=1,ncol
	      jj=j+ncol*(k-1)
	      yprime(jj)=dq(j,k)
            if (pfk.lt.0.9999) yprime(jj+nac*ncol)=dp(j,k)
  601       continue
  603  continue

       t=t

       return
       end

c    -----for divpag----------
       subroutine fcnj(nn,t,y,dy)
	 IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	 parameter (ndim4=29)
 	 parameter (ndim6=13)
	      
       dimension y(nn),dy(nn,nn)
       dimension acdose(ndim6),acdia(ndim6),radi(ndim6)
       dimension wr(ndim4),br(ndim4,ndim4)
       dimension q(ndim4,ndim6),p(ndim4,ndim6),cx(ndim4,ndim6)
       dimension qq(ndim4,ndim6,ndim4,ndim6)
     &          ,pp(ndim4,ndim6,ndim4,ndim6)
     &          ,qp(ndim4,ndim6,ndim4,ndim6)
     &          ,pq(ndim4,ndim6,ndim4,ndim6)
     	  	 
	 common/b1/nac
	 common/b2/acdose,acdia,radi
       common/b3/ps,rfk,rn,pfk,rkf,rds,rdp,rks,co,ceff,rceff
       common/b4/ncol
       common/b5/wr,br

       external sset

	 call sset (nn**2, 0.0, dy, 1)
   
       do 110 i=1,nn
	 do 111 j=1,nn
	 dy(i,j)=0.0
 111   continue
 110   continue

 	    do 120 k=1,nac
		  do 121 j=1,ncol
	      jj=j+ncol*(k-1)
	      q(j,k)=y(jj)
	      if (pfk.lt.0.9999) p(j,k)=y(jj+nac*ncol)
  121       continue
  120       continue  
 
	 ncol1=ncol-1

	 do 201 m=1,nac

	 do 301 i=1,ncol1
	 do 302	n=1,nac
       do 303 j=1,ncol

       if(m.eq.n) then
c-----------------------------------------------------
	 qrn=dsign(abs(q(j,m))**(rn-1.0),q(j,m))
       qq(i,m,j,n)=rds/radi(m)**2*br(i,j)+
     &  +rdp/radi(m)**2/ps*rn*br(i,j)*qrn/rfk**rn
c-----------------------------------------------------
       else
       qq(i,m,j,n)=0.0
       end if

  303  continue
  302  continue
  301  continue

	 do 311	n=1,nac
       do 312 j=1,ncol1

       if(m.eq.n) then
c-----------------------------------------------------
	 qrn=dsign(abs(q(j,m))**(rn-1.0),q(j,m))
 	 qq(ncol,m,j,n)
     &   =rkf*(6.0/acdia(m))/ps/wr(ncol)*(-wr(j)*acdose(m))
       do 313 k=1,ncol1
       qq(ncol,m,j,n)=qq(ncol,m,j,n)
     &               -(rds+rdp/ps*rn*qrn/rfk**rn)
     &                /wr(ncol)/radi(m)**2*wr(k)*br(k,j)
  313  continue
c-----------------------------------------------------
       else
	 qq(ncol,m,j,n)
     &   =rkf*(6.0/acdia(m))/ps/wr(ncol)*(-wr(j)*acdose(n))
	 end if

  312  continue
  311  continue

	 do 321	n=1,nac
       if(m.eq.n) then
c-----------------------------------------------------
	 qrn=dsign(abs(q(ncol,m))**(rn-1.0),q(ncol,m))
	 qq(ncol,m,ncol,n)=
     &   rkf*(6.0/acdia(m))/3.0/ps/wr(ncol)
     &  *(-3.0*acdose(m)*wr(ncol)-rn/rfk**rn*qrn)

       do 322 k=1,ncol1
       qq(ncol,m,ncol,n)=qq(ncol,m,ncol,n)
     &               -(rds+rdp/ps*rn*qrn/rfk**rn)
     &                /wr(ncol)/radi(m)**2*wr(k)*br(k,ncol)
  322  continue
c-----------------------------------------------------
       else
	 qq(ncol,m,ncol,n)
     &   =rkf*(6.0/acdia(m))/ps/wr(ncol)*(-wr(ncol)*acdose(n))
	 end if

  321   continue

  201   continue

       if (pfk.lt.0.9999) then

	 do 401 m=1,nac

	 do 501 i=1,ncol
	 do 502	n=1,nac
       do 503 j=1,ncol
       qp(i,m,j,n)=0.0
       pq(i,m,j,n)=0.0
       pp(i,m,j,n)=0.0
  503  continue
  502  continue
       qq(i,m,i,m)=qq(i,m,i,m)-rks/pfk
	 qp(i,m,i,m)=            rks/(1.0-pfk)
	 pq(i,m,i,m)=            rks/pfk
	 pp(i,m,i,m)=           -rks/(1.0-pfk)
  501  continue

	 do 511	n=1,nac
       do 512 j=1,ncol
	 qp(ncol,m,j,n)=qp(ncol,m,j,n)
     &   +rkf*(6.0/acdia(m))/ps/wr(ncol)*(-wr(j)*acdose(n))
  512  continue
  511  continue

  401   continue

       else
	 end if

   	 do 601 m=1,nac
	      do 602 i=1,ncol
       do 603 n=1,nac
	      do 604 j=1,ncol
	      ii=i+ncol*(m-1)
	      jj=j+ncol*(n-1)
	      dy(ii,jj)=qq(i,m,j,n)
  604  continue
  603  continue
  602  continue
  601  continue

	 if (pfk.lt.0.9999) then
   	 do 701 m=1,nac
	      do 702 i=1,ncol
       do 703 n=1,nac
	      do 704 j=1,ncol
	      ii=i+ncol*(m-1)
	      jj=j+ncol*(n-1)
	      dy(ii,         jj+nac*ncol)=qp(i,m,j,n)
	      dy(ii+nac*ncol,jj         )=pq(i,m,j,n)
	      dy(ii+nac*ncol,jj+nac*ncol)=pp(i,m,j,n)
  704  continue
  703  continue
  702  continue
  701  continue
       else
	 end if

       t=t

	 return
	 end

	                       